﻿using System;
using System.Drawing;
using System.Configuration;
using DevExpress.XtraEditors;

using DotSpatial.Controls;
using DotSpatial.Data;
using DotSpatial.Symbology;
using DotSpatial.Plugins.OverviewMap.Properties;
using DotSpatial.Controls.Docking;
using System.Windows.Forms;
using GeoAPI.Geometries;
using DotSpatial.NTSExtension;

namespace DotSpatial.Plugins.OverviewMap
{
    public class ResizeableControl
    {
        private Control withEventsField_mControl;
        private Control mControl
        {
            get { return withEventsField_mControl; }
            set
            {
                if (withEventsField_mControl != null)
                {
                    withEventsField_mControl.MouseDown -= mControl_MouseDown;
                    withEventsField_mControl.MouseUp -= mControl_MouseUp;
                    withEventsField_mControl.MouseMove -= mControl_MouseMove;
                    withEventsField_mControl.MouseLeave -= mControl_MouseLeave;
                }
                withEventsField_mControl = value;
                if (withEventsField_mControl != null)
                {
                    withEventsField_mControl.MouseDown += mControl_MouseDown;
                    withEventsField_mControl.MouseUp += mControl_MouseUp;
                    withEventsField_mControl.MouseMove += mControl_MouseMove;
                    withEventsField_mControl.MouseLeave += mControl_MouseLeave;
                }
            }
        }
        private bool mMouseDown = false;
        private EdgeEnum mEdge = EdgeEnum.None;
        private int mWidth = 4;

        private bool mOutlineDrawn = false;
        private enum EdgeEnum
        {
            None,
            Left,
            Bottom,
            BottomLeft
        }

        public ResizeableControl(Control Control)
        {
            mControl = Control;
        }


        private void mControl_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                mMouseDown = true;
            }
        }


        private void mControl_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            mMouseDown = false;
        }


        private void mControl_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            Control c = (Control)sender;

            if (mMouseDown & mEdge != EdgeEnum.None)
            {
                c.SuspendLayout();
                switch (mEdge)
                {
                    case EdgeEnum.BottomLeft:
                        c.SetBounds(c.Left + e.X, c.Top, c.Width - e.X, c.Height - (c.Height - e.Y));
                        break;
                    case EdgeEnum.Left:
                        c.SetBounds(c.Left + e.X, c.Top, c.Width - e.X, c.Height);
                        break;
                    case EdgeEnum.Bottom:
                        c.SetBounds(c.Left, c.Top, c.Width, c.Height - (c.Height - e.Y));
                        break;
                }
                c.ResumeLayout();
            }
            else
            {
                if (e.X <= (mWidth * 4) & e.Y > c.Height - (mWidth + 1))
                {
                    //top left corner
                    c.Cursor = Cursors.SizeNESW;
                    mEdge = EdgeEnum.BottomLeft;
                }
                else if( e.X <= mWidth)
                {
                    //left edge
                    c.Cursor = Cursors.SizeWE;
                    mEdge = EdgeEnum.Left;
                }
                else if( e.Y > c.Height - (mWidth + 1))
                {
                    //bottom edge
                    c.Cursor = Cursors.SizeNS;
                    mEdge = EdgeEnum.Bottom;
                }
                else
                {
                    //no edge
                    c.Cursor = Cursors.Default;
                    mEdge = EdgeEnum.None;
                }
            }
        }


        private void mControl_MouseLeave(object sender, System.EventArgs e)
        {
            Control c = (Control)sender;
            mEdge = EdgeEnum.None;
            c.Refresh();
        }

    }
}
